/****************************************************************************************
***** Macro qui assigne l'indice de dfavorisation de 2016 du Qubec ainsi que ses 	*****
***** dclinaisons locales (RSS, RLS, CLSC) selon les dcoupages pr-rforme 2015. 	*****
***** L'assignation est ralise  partir du code postal (obligatoire) et du code	*****
***** municipal (facultatif). La macro est base sur une version simplifie du		*****
***** Fichier de conversion des codes postaux (FCCP+) de Statistique Canada et fait *****
***** appel aux trois fichiers de conversion suivants:								*****
*****  - Fichier de codes postaux uniques (PCCFUNIQ)                                *****
*****  - Fichier de codes postaux non uniques (PCCFDUPS)                           	*****
*****  - Fichier de codes postaux non uniques avec pondration connue (WCF)         *****
*****  cela s'ajoute un quatrime fichier (MUNIC) qui effectue l'historique des	*****
***** codes municipaux au Canada de 1991  2016.									*****
*****																				***** 
***** Les quatre paramtres de la macro sont:                                       *****
*****  - IN: nom du fichier SAS en entre auquel on dsire ajouter l'indice de   	*****
*****        dfavorisation de 2016.						    					*****
*****  - PCODE: nom de la variable du code postal dans le fichier d'entre.    		*****
*****  - MUNIC: nom de la variable du code municipal dans le fichier d'entre. Par  *****
*****           dfaut, la valeur 0 (zro) signifie que l'assignation est ralise  *****
*****           seulement  partir du code postal.                        			*****
*****  - OUT: nom du fichier SAS de sortie, qui sera identique au fichier d'entre 	*****
*****         mais bonifi des deux nouvelles variables.      						*****
*****																				*****
***** Auteur: Denis Hamel, INSPQ                                    				*****
*****         Philippe Gamache, INSPQ                               				*****
*****         Christine Blaser, INSPQ                               				*****
***** Dernire modification: 7 dcembre 2020		                       			*****
*****************************************************************************************/


/****************************************************************************************/
/* 1. tablir le chemin daccs du rpertoire o se trouvent les 4 fichiers de donnes    */
/* Par exemple: ;
%let repertoire = D:\Projets;


/****************************************************************************************/
/* 2. Dterminer les quatre paramtres (par dfaut, munic = 0)						    */
%let in = fichier_depart;
%let pcode = codepostal;
%let munic = 0;
%let out = fichier_assigne;


/****************************************************************************************/
/* 3. Excution de la macro. Ne rien changer  partir d'ici						    	*/
%macro assignation16(in=, pcode=, munic=, out=);

filename fcp "&repertoire\fcp.txt";
filename pccfuniq "&repertoire\fccpindiceuniq.txt";
filename pccfdups "&repertoire\fccpindicedouble.txt";
filename munic "&repertoire\munic2016.txt";

libname i2016 "&repertoire";

data i2016.wcf;
  infile fcp;
  input munic 1-12 pcode $ 13-18 propmunic 19-23 .2 propcp 5.2 first 29 
        QuintMat 30 QuintSoc 31 QuintMatRSS 32 QuintSocRSS 33 QuartMatRSS 34 QuartSocRSS 35 QuintMatRTS 36 QuintSocRTS 37 QuartMatRTS 38 QuartSocRTS 39 
        QuintMatRLS 40 QuintSocRLS 41 QuartMatRLS 42 QuartSocRLS 43 QuintMatCLSC 44 QuintSocCLSC 45 QuartMatCLSC 46 QuartSocCLSC 47 
        zone 49 RSS $ 50-51 RTS $ 52-53 RLS $ 54-57 CLSC $ 58-62 CSS 63 propmunic_CLSC 5.2 propcp_CLSC 5.2;
run;

data i2016.pccfuniq;
  infile pccfuniq;
  input munic 1-12 pcode $ 13-18 propmunic 19-23 .2 propcp 5.2 first 29 
        QuintMat 30 QuintSoc 31 QuintMatRSS 32 QuintSocRSS 33 QuartMatRSS 34 QuartSocRSS 35 QuintMatRTS 36 QuintSocRTS 37 QuartMatRTS 38 QuartSocRTS 39 
        QuintMatRLS 40 QuintSocRLS 41 QuartMatRLS 42 QuartSocRLS 43 QuintMatCLSC 44 QuintSocCLSC 45 QuartMatCLSC 46 QuartSocCLSC 47 
        zone 49 RSS $ 50-51 RTS $ 52-53 RLS $ 54-57 CLSC $ 58-62 CSS 63 propmunic_CLSC 5.2 propcp_CLSC 5.2;
run;

data i2016.pccfdups;
  infile pccfdups;
  input munic 1-12 pcode $ 13-18 propmunic 19-23 .2 propcp 5.2 first 29 
        QuintMat 30 QuintSoc 31 QuintMatRSS 32 QuintSocRSS 33 QuartMatRSS 34 QuartSocRSS 35 QuintMatRTS 36 QuintSocRTS 37 QuartMatRTS 38 QuartSocRTS 39 
        QuintMatRLS 40 QuintSocRLS 41 QuartMatRLS 42 QuartSocRLS 43 QuintMatCLSC 44 QuintSocCLSC 45 QuartMatCLSC 46 QuartSocCLSC 47 
        zone 49 RSS $ 50-51 RTS $ 52-53 RLS $ 54-57 CLSC $ 58-62 CSS 63 propmunic_CLSC 5.2 propcp_CLSC 5.2;
run;

data i2016.munic;
  infile munic;
  input munic 1-7 mun2016 8-14;
run;

data wcf;
  set i2016.wcf;
  nuniq = _n_;
run;

data pccfuniq;
  set i2016.pccfuniq;
  nuniq = _n_;
run;

data pccfdups;
  set i2016.pccfdups;
  nuniq = _n_;
run;

data fichin;
  %if &munic ne 0 %then %do; 
    length munic 8.; 
  %end;
  set &in;
  numero=_n_;
  nba=ranuni(766);
  pcode=compress(&pcode);
  %if &munic ne 0 %then %do; 
    munic=2400000+mod(&munic,2400000); 
  %end;
run;

%presence(var=quintmat);

%if &presence ne 0 %then %do;
data fichin;
  set fichin;
  where zone in (.,0);
run;
%end;

data fichin;
  set fichin;
  keep numero nba pcode %if &munic ne 0 %then %do; munic %end;;
run;


/****************************************************************************************/
/* Assignation de l'indice  partir du code postal et de la municipalit (si munic ne 0)*/

%if &munic ne 0 %then %do;

proc sort data=i2016.munic;by munic;run;
proc sort data=fichin;by munic;run;

data fichin;
  merge i2016.munic(in=in2) fichin(in=in1);
  by munic;
  if in1*in2 then munic=mun2016;
  else if in2 then delete;
run;

/***** Avec PCCFUNIQ *****/
/*** Qubec et RSS ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.munic, fichin.numero, fichin.nba, pccfuniq.first, pccfuniq.propmunic_CLSC, 
         pccfuniq.nuniq, pccfuniq.zone, pccfuniq.css, pccfuniq.QuintMat, pccfuniq.QuintSoc, 
         pccfuniq.RSS, pccfuniq.quintmatRSS, pccfuniq.quintsocRSS, pccfuniq.quartmatRSS, pccfuniq.quartsocRSS,
         pccfuniq.RTS, pccfuniq.quintmatRTS, pccfuniq.quintsocRTS, pccfuniq.quartmatRTS, pccfuniq.quartsocRTS, 
         pccfuniq.RLS, pccfuniq.quintmatRLS, pccfuniq.quintsocRLS, pccfuniq.quartmatRLS, pccfuniq.quartsocRLS, 
         pccfuniq.CLSC, pccfuniq.quintmatCLSC, pccfuniq.quintsocCLSC, pccfuniq.quartmatCLSC, pccfuniq.quartsocCLSC
  from fichin, pccfuniq
  where fichin.pcode=pccfuniq.pcode and fichin.munic=pccfuniq.munic
  order by numero,nuniq;
quit;

proc sort data=test;by pcode munic numero;run;

 /*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test1;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic_CLSC/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

proc sort data=test1;by numero;run;
proc sort data=fichin;by numero;run; 

data appmunic1 nonapp1;
  merge fichin(in=in1) test1(in=in2a);
  by numero;
  if in1*in2a then output appmunic1;
  else if in1 then output nonapp1;
run;

/***** Avec WCF *****/
proc sql;
  create table test as
  select fichin.pcode, fichin.munic, fichin.numero, fichin.nba, wcf.first, wcf.propmunic_CLSC,
         wcf.nuniq, wcf.css, wcf.zone, wcf.QuintMat, wcf.QuintSoc, 
         wcf.RSS, wcf.quintmatRSS, wcf.quintsocRSS, wcf.quartmatRSS, wcf.quartsocRSS, 
         wcf.RTS, wcf.quintmatRTS, wcf.quintsocRTS, wcf.quartmatRTS, wcf.quartsocRTS, 
         wcf.RLS, wcf.quintmatRLS, wcf.quintsocRLS, wcf.quartmatRLS, wcf.quartsocRLS, 
         wcf.CLSC, wcf.quintmatCLSC, wcf.quintsocCLSC, wcf.quartmatCLSC, wcf.quartsocCLSC
  from fichin, wcf
  where fichin.pcode=wcf.pcode and fichin.munic=wcf.munic
  order by numero,nuniq;
quit;

proc sort data=test;by pcode munic numero;run;

 /*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test1;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic_CLSC/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

proc sort data=test1;by numero;run;
proc sort data=nonapp1;by numero;run; 

data appmunic2 nonapp2;
  merge nonapp1(in=in1) test1(in=in2a);
  by numero;
  if in1*in2a then output appmunic2;
  else if in1 then output nonapp2;
run;

/***** Avec PCCFDUPS *****/
/*** Qubec et RSS ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.munic, fichin.numero, fichin.nba, pccfdups.first, pccfdups.propmunic_CLSC, 
         pccfdups.nuniq, pccfdups.zone, pccfdups.css, pccfdups.QuintMat, pccfdups.QuintSoc, 
         pccfdups.RSS, pccfdups.quintmatRSS, pccfdups.quintsocRSS, pccfdups.quartmatRSS, pccfdups.quartsocRSS, 
         pccfdups.RTS, pccfdups.quintmatRTS, pccfdups.quintsocRTS, pccfdups.quartmatRTS, pccfdups.quartsocRTS, 
         pccfdups.RLS, pccfdups.quintmatRLS, pccfdups.quintsocRLS, pccfdups.quartmatRLS, pccfdups.quartsocRLS, 
         pccfdups.CLSC, pccfdups.quintmatCLSC, pccfdups.quintsocCLSC, pccfdups.quartmatCLSC, pccfdups.quartsocCLSC
  from fichin, pccfdups
  where fichin.pcode=pccfdups.pcode and fichin.munic=pccfdups.munic
  order by numero,nuniq;
quit;

proc sort data=test;by pcode munic numero;run;

 /*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test1;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic_CLSC/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

proc sort data=test1;by numero;run;

proc sort data=nonapp2;by numero;run; 

data appmunic3 fichin;
  merge nonapp2(in=in1) test1(in=in2a);
  by numero;
  if in1*in2a then output appmunic3;
  else if in1 then output fichin;
run;

%end;

/****************************************************************************************/
/* Assignation de l'indice  partir du code postal seulement							*/

/***** Avec PCCFUNIQ *****/
proc sql;
  create table test as
  select fichin.pcode, fichin.numero, fichin.nba, pccfuniq.first, pccfuniq.propcp_CLSC,
         pccfuniq.nuniq, pccfuniq.zone, pccfuniq.css, pccfuniq.QuintMat, pccfuniq.QuintSoc, 
         pccfuniq.RSS, pccfuniq.quintmatRSS, pccfuniq.quintsocRSS, pccfuniq.quartmatRSS, pccfuniq.quartsocRSS, 
         pccfuniq.RTS, pccfuniq.quintmatRTS, pccfuniq.quintsocRTS, pccfuniq.quartmatRTS, pccfuniq.quartsocRTS, 
         pccfuniq.RLS, pccfuniq.quintmatRLS, pccfuniq.quintsocRLS, pccfuniq.quartmatRLS, pccfuniq.quartsocRLS, 
         pccfuniq.CLSC, pccfuniq.quintmatCLSC, pccfuniq.quintsocCLSC, pccfuniq.quartmatCLSC, pccfuniq.quartsocCLSC
  from fichin, pccfuniq
  where fichin.pcode=pccfuniq.pcode
  order by numero,nuniq;
quit;

proc sort data=test;by pcode numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test1;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp_CLSC/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

proc sort data=test1;by numero;run;
proc sort data=fichin;by numero;run; 

data appcp1 nonapp3;
  merge fichin(in=in1) test1(in=in2a);
  by numero;
  if in1*in2a then output appcp1;
  else if in1 then output nonapp3;
run;

/***** Avec WCF *****/
proc sql;
  create table test as
  select fichin.pcode, fichin.numero, fichin.nba, wcf.first, wcf.propcp_CLSC, 
         wcf.css, wcf.nuniq, wcf.zone, wcf.QuintMat, wcf.QuintSoc, 
         wcf.RSS, wcf.quintmatRSS, wcf.quintsocRSS,  wcf.quartmatRSS, wcf.quartsocRSS,
         wcf.RTS, wcf.quintmatRTS, wcf.quintsocRTS,  wcf.quartmatRTS, wcf.quartsocRTS, 
         wcf.RLS, wcf.quintmatRLS, wcf.quintsocRLS,  wcf.quartmatRLS, wcf.quartsocRLS, 
         wcf.CLSC, wcf.quintmatCLSC, wcf.quintsocCLSC,  wcf.quartmatCLSC, wcf.quartsocCLSC
  from fichin, wcf
  where fichin.pcode=wcf.pcode
  order by numero,nuniq;
quit;

proc sort data=test;by pcode numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test1;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp_CLSC/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

proc sort data=test1;by numero;run;
proc sort data=nonapp3;by numero;run; 

data appcp2 nonapp4;
  merge nonapp3(in=in1) test1(in=in2a);
  by numero;
  if in1*in2a then output appcp2;
  else if in1 then output nonapp4;
run;

/***** Avec PCCFDUPS *****/
proc sql;
  create table test as
  select fichin.pcode, fichin.numero, fichin.nba, pccfdups.first, pccfdups.propcp_CLSC,
         pccfdups.css, pccfdups.nuniq, pccfdups.zone, pccfdups.QuintMat, pccfdups.QuintSoc, 
         pccfdups.RSS, pccfdups.quintmatRSS, pccfdups.quintsocRSS, pccfdups.quartmatRSS, pccfdups.quartsocRSS, 
         pccfdups.RTS, pccfdups.quintmatRTS, pccfdups.quintsocRTS, pccfdups.quartmatRTS, pccfdups.quartsocRTS, 
         pccfdups.RLS, pccfdups.quintmatRLS, pccfdups.quintsocRLS, pccfdups.quartmatRLS, pccfdups.quartsocRLS, 
         pccfdups.CLSC, pccfdups.quintmatCLSC, pccfdups.quintsocCLSC, pccfdups.quartmatCLSC, pccfdups.quartsocCLSC   
  from fichin, pccfdups
  where fichin.pcode=pccfdups.pcode
  order by numero,nuniq;
quit;

proc sort data=test;by pcode numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test1;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp_CLSC/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

proc sort data=test1;by numero;run;
proc sort data=nonapp4;by numero;run; 

data appcp3 nonapp5;
  merge nonapp4(in=in1) test1(in=in2a);
  by numero;
  if in1*in2a then output appcp3;
  else if in1 then output nonapp5;
run;


data tout;
  set %if &munic ne 0 %then %do; appmunic1(in=in1) appmunic2(in=in2) appmunic3(in=in3) %end; 
                                 appcp1(in=in4) appcp2(in=in5) appcp3(in=in6) nonapp5(in=in7);
  if in7 then direct=0;
  if in5 then do; 
    direct=1;
  end;

  %if &munic ne 0 %then %do; 
     if in1 then base=1;
     else if in2 then do; 
       direct=1;base=2;
     end;
	 else if in3 then base=3;
   %end;
   if in4 then base=1;
   else if in5 then base=2;
   else if in6 then base=3;

   if in7=1 then do;
     zone = 0;css=0;QuintMat = 0;QuintSoc = 0;
	 quartmatRSS=0;quartsocRSS=0;quintmatRSS=0; quintsocRSS=0;
     quintmatRTS=0;quintsocRTS=0;quartmatRTS=0;quartsocRTS=0;
     quintmatRLS=0;quintsocRLS=0;quartmatRLS=0;quartsocRLS=0;
     quintmatCLSC=0;quintsocCLSC=0;quartmatCLSC=0;quartsocCLSC=0;
   end;
   keep numero base zone css quintmat quintsoc 
        RSS quartmatRSS quartsocRSS quintmatRSS quintsocRSS 
        RTS quintmatRTS quintsocRTS quartmatRTS quartsocRTS 
        RLS quintmatRLS quintsocRLS quartmatRLS quartsocRLS 
        CLSC quintmatCLSC quintsocCLSC quartmatCLSC quartsocCLSC;
run;

proc sort; by numero; run; 

data fichin;
  set &in;
  numero=_n_;
run;

data tout;
  retain zone css QuintMat QuintSoc 
         RSS QuintMatRSS QuintSocRSS QuartMatRSS QuartSocRSS 
         RTS QuintMatRTS QuintSocRTS QuartMatRTS QuartSocRTS
         RLS QuintMatRLS QuintSocRLS QuartMatRLS QuartSocRLS 
         CLSC QuintMatCLSC QuintSocCLSC QuartMatCLSC QuartSocCLSC;
  set tout;
run;

data &out;
  merge fichin tout;
  by numero;
  DATE = '20201220';
  *format groupe fgroupe. base fbase.;
  drop numero;
run;

proc datasets library=work;
  delete corr2 wcf2 pccfuniq2 pccfdups2 test test1 fichin tout appmunic1 appmunic2 appmunic3 appmunic4 appcp1 appcp2 appcp3 appcp4 nonapp1 nonapp2 
         nonapp3 nonapp4 nonapp5 nonapp6 nonapp7 pccfdup2; 
quit;
%mend assignation16;

%macro presence(var=);
%global presence;
%let dsid=%sysfunc(open(fichin,i));
    %if &dsid>0 %then
       %do;
	     %let presence=%sysfunc(varnum(&dsid,&var));
		 /*** la macrovariable presence = 0 si la variable est absente du fichier ****/
	   %end;
%put &presence;
%let rc = %sysfunc(close(&dsid));
%mend presence;


/***** Exemple *****/
%let in = deces;
%let pcode = CPRESDCD;
%let munic = CMUNRESD;
%let out = deces_assigne;

%assignation16(in=&in,pcode=&pcode,munic=&munic,out=&out);
